from typing import overload
from enum import IntEnum
import typing

import QuantConnect.Orders
import QuantConnect.Orders.Serialization
import QuantConnect.Util
import System


class SerializedOrderEvent(System.Object):
    """Data transfer object used for serializing an OrderEvent that was just generated by an algorithm"""

    @property
    def id(self) -> str:
        """The unique order event id"""
        ...

    @property
    def algorithm_id(self) -> str:
        """Algorithm Id, BacktestId or DeployId"""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @property
    def order_id(self) -> int:
        """Id of the order this event comes from."""
        ...

    @order_id.setter
    def order_id(self, value: int) -> None:
        ...

    @property
    def order_event_id(self) -> int:
        """The unique order event id for each order"""
        ...

    @order_event_id.setter
    def order_event_id(self, value: int) -> None:
        ...

    @property
    def symbol(self) -> str:
        """Easy access to the order symbol associated with this event."""
        ...

    @symbol.setter
    def symbol(self, value: str) -> None:
        ...

    @property
    def symbol_value(self) -> str:
        """The mapped symbol value"""
        ...

    @symbol_value.setter
    def symbol_value(self, value: str) -> None:
        ...

    @property
    def symbol_permtick(self) -> str:
        """The symbols permanent ticker. For equities, by convention this is the first ticker symbol for which the security traded"""
        ...

    @symbol_permtick.setter
    def symbol_permtick(self, value: str) -> None:
        ...

    @property
    def time(self) -> float:
        """The time of this event in unix timestamp"""
        ...

    @time.setter
    def time(self, value: float) -> None:
        ...

    @property
    def status(self) -> QuantConnect.Orders.OrderStatus:
        """Status message of the order."""
        ...

    @status.setter
    def status(self, value: QuantConnect.Orders.OrderStatus) -> None:
        ...

    @property
    def order_fee_amount(self) -> typing.Optional[float]:
        """The fee amount associated with the order"""
        ...

    @order_fee_amount.setter
    def order_fee_amount(self, value: typing.Optional[float]) -> None:
        ...

    @property
    def order_fee_currency(self) -> str:
        """The fee currency associated with the order"""
        ...

    @order_fee_currency.setter
    def order_fee_currency(self, value: str) -> None:
        ...

    @property
    def fill_price(self) -> float:
        """Fill price information about the order"""
        ...

    @fill_price.setter
    def fill_price(self, value: float) -> None:
        ...

    @property
    def fill_price_currency(self) -> str:
        """Currency for the fill price"""
        ...

    @fill_price_currency.setter
    def fill_price_currency(self, value: str) -> None:
        ...

    @property
    def fill_quantity(self) -> float:
        """Number of shares of the order that was filled in this event."""
        ...

    @fill_quantity.setter
    def fill_quantity(self, value: float) -> None:
        ...

    @property
    def direction(self) -> QuantConnect.Orders.OrderDirection:
        """Order direction."""
        ...

    @direction.setter
    def direction(self, value: QuantConnect.Orders.OrderDirection) -> None:
        ...

    @property
    def message(self) -> str:
        """Any message from the exchange."""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    @property
    def is_assignment(self) -> bool:
        """True if the order event is an assignment"""
        ...

    @is_assignment.setter
    def is_assignment(self, value: bool) -> None:
        ...

    @property
    def quantity(self) -> float:
        """The current order quantity"""
        ...

    @quantity.setter
    def quantity(self, value: float) -> None:
        ...

    @property
    def stop_price(self) -> typing.Optional[float]:
        """The current stop price"""
        ...

    @stop_price.setter
    def stop_price(self, value: typing.Optional[float]) -> None:
        ...

    @property
    def limit_price(self) -> typing.Optional[float]:
        """The current limit price"""
        ...

    @limit_price.setter
    def limit_price(self, value: typing.Optional[float]) -> None:
        ...

    @property
    def is_in_the_money(self) -> bool:
        """True if the order event's option is In-The-Money (ITM)"""
        ...

    @is_in_the_money.setter
    def is_in_the_money(self, value: bool) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Empty constructor required for JSON converter."""
        ...

    @overload
    def __init__(self, order_event: QuantConnect.Orders.OrderEvent, algorithm_id: str) -> None:
        """Creates a new instances based on the provided order event and algorithm Id"""
        ...


class OrderEventJsonConverter(QuantConnect.Util.TypeChangeJsonConverter[QuantConnect.Orders.OrderEvent, QuantConnect.Orders.Serialization.SerializedOrderEvent]):
    """Defines how OrderEvents should be serialized to json"""

    @property
    def populate_properties(self) -> bool:
        """
        True will populate TResult object returned by convert(SerializedOrderEvent) with json properties
        
        
        This codeEntityType is protected.
        """
        ...

    def __init__(self, algorithm_id: str = None) -> None:
        """
        Creates a new instance
        
        :param algorithm_id: The associated algorithm id, required when serializing
        """
        ...

    @overload
    def convert(self, value: QuantConnect.Orders.OrderEvent) -> QuantConnect.Orders.Serialization.SerializedOrderEvent:
        """
        Convert the input value to a value to be serialzied
        
        
        This codeEntityType is protected.
        
        :param value: The input value to be converted before serialziation
        :returns: A new instance of TResult that is to be serialzied.
        """
        ...

    @overload
    def convert(self, value: QuantConnect.Orders.Serialization.SerializedOrderEvent) -> QuantConnect.Orders.OrderEvent:
        """
        Converts the input value to be deserialized
        
        
        This codeEntityType is protected.
        
        :param value: The deserialized value that needs to be converted to OrderEvent
        :returns: The converted value.
        """
        ...





from typing import overload
from enum import IntEnum
import datetime
import typing

import QuantConnect
import QuantConnect.Data
import QuantConnect.Data.Custom.IconicTypes
import QuantConnect.Data.Market


class IndexedLinkedData(QuantConnect.Data.IndexedBaseData):
    """
    Data type that is indexed, i.e. a file that points to another file containing the contents
    we're looking for in a Symbol.
    """

    @property
    def count(self) -> int:
        """Example data property"""
        ...

    @count.setter
    def count(self, value: int) -> None:
        ...

    def data_time_zone(self) -> typing.Any:
        """
        Set the data time zone to UTC
        
        :returns: Time zone as UTC.
        """
        ...

    def default_resolution(self) -> QuantConnect.Resolution:
        """
        Sets the default resolution to Second
        
        :returns: Resolution.Second.
        """
        ...

    def get_source(self, config: QuantConnect.Data.SubscriptionDataConfig, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.SubscriptionDataSource:
        """
        Gets the source of the index file
        
        :param config: Configuration object
        :param date: Date of this source file
        :param is_live_mode: Is live mode
        :returns: SubscriptionDataSource indicating where data is located and how it's stored.
        """
        ...

    def get_source_for_an_index(self, config: QuantConnect.Data.SubscriptionDataConfig, date: datetime.datetime, index: str, is_live_mode: bool) -> QuantConnect.Data.SubscriptionDataSource:
        """
        Determines the actual source from an index contained within a ticker folder
        
        :param config: Subscription configuration
        :param date: Date
        :param index: File to load data from
        :param is_live_mode: Is live mode
        :returns: SubscriptionDataSource pointing to the article.
        """
        ...

    def is_sparse_data(self) -> bool:
        """
        Indicates whether the data source is sparse.
        If false, it will disable missing file logging.
        
        :returns: true.
        """
        ...

    def reader(self, config: QuantConnect.Data.SubscriptionDataConfig, line: str, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.BaseData:
        """
        Creates an instance from a line of JSON containing article information read from the `content` directory
        
        :param config: Subscription configuration
        :param line: Line of data
        :param date: Date
        :param is_live_mode: Is live mode
        """
        ...

    def requires_mapping(self) -> bool:
        """
        Indicates whether the data source can undergo
        rename events/is tied to equities.
        
        :returns: true.
        """
        ...

    def supported_resolutions(self) -> typing.List[QuantConnect.Resolution]:
        """
        Gets a list of all the supported Resolutions
        
        :returns: All resolutions.
        """
        ...


class IndexedLinkedData2(QuantConnect.Data.IndexedBaseData):
    """
    Data type that is indexed, i.e. a file that points to another file containing the contents
    we're looking for in a Symbol.
    """

    @property
    def count(self) -> int:
        """Example data property"""
        ...

    @count.setter
    def count(self, value: int) -> None:
        ...

    def data_time_zone(self) -> typing.Any:
        """
        Set the data time zone to UTC
        
        :returns: Time zone as UTC.
        """
        ...

    def default_resolution(self) -> QuantConnect.Resolution:
        """
        Sets the default resolution to Second
        
        :returns: Resolution.Second.
        """
        ...

    def get_source(self, config: QuantConnect.Data.SubscriptionDataConfig, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.SubscriptionDataSource:
        """
        Gets the source of the index file
        
        :param config: Configuration object
        :param date: Date of this source file
        :param is_live_mode: Is live mode
        :returns: SubscriptionDataSource indicating where data is located and how it's stored.
        """
        ...

    def get_source_for_an_index(self, config: QuantConnect.Data.SubscriptionDataConfig, date: datetime.datetime, index: str, is_live_mode: bool) -> QuantConnect.Data.SubscriptionDataSource:
        """
        Determines the actual source from an index contained within a ticker folder
        
        :param config: Subscription configuration
        :param date: Date
        :param index: File to load data from
        :param is_live_mode: Is live mode
        :returns: SubscriptionDataSource pointing to the article.
        """
        ...

    def is_sparse_data(self) -> bool:
        """
        Indicates whether the data source is sparse.
        If false, it will disable missing file logging.
        
        :returns: true.
        """
        ...

    def reader(self, config: QuantConnect.Data.SubscriptionDataConfig, line: str, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.BaseData:
        """
        Creates an instance from a line of JSON containing article information read from the `content` directory
        
        :param config: Subscription configuration
        :param line: Line of data
        :param date: Date
        :param is_live_mode: Is live mode
        """
        ...

    def requires_mapping(self) -> bool:
        """
        Indicates whether the data source can undergo
        rename events/is tied to equities.
        
        :returns: true.
        """
        ...

    def supported_resolutions(self) -> typing.List[QuantConnect.Resolution]:
        """
        Gets a list of all the supported Resolutions
        
        :returns: All resolutions.
        """
        ...


class UnlinkedData(QuantConnect.Data.BaseData):
    """Data source that is unlinked (no mapping) and takes any ticker when calling AddData"""

    any_ticker: bool
    """If true, we accept any ticker from the AddData call"""

    @property
    def ticker(self) -> str:
        """Example data"""
        ...

    @ticker.setter
    def ticker(self, value: str) -> None:
        ...

    def data_time_zone(self) -> typing.Any:
        """
        Set the data time zone to UTC
        
        :returns: Time zone as UTC.
        """
        ...

    def default_resolution(self) -> QuantConnect.Resolution:
        """
        Sets the default resolution to Second
        
        :returns: Resolution.Second.
        """
        ...

    def get_source(self, config: QuantConnect.Data.SubscriptionDataConfig, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.SubscriptionDataSource:
        """
        Return the path string source of the file. This will be converted to a stream
        
        :param config: Configuration object
        :param date: Date of this source file
        :param is_live_mode: true if we're in live mode, false for backtesting mode
        :returns: String path of source file.
        """
        ...

    def is_sparse_data(self) -> bool:
        """
        Indicates whether the data source is sparse.
        If false, it will disable missing file logging.
        
        :returns: true.
        """
        ...

    def reader(self, config: QuantConnect.Data.SubscriptionDataConfig, line: str, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.BaseData:
        """
        Creates UnlinkedData objects using the subscription data config setup as well as the date.
        
        :param config: Subscription data config setup object
        :param line: Line of the source document
        :param date: Date of the requested data
        :param is_live_mode: true if we're in live mode, false for backtesting mode
        :returns: Instance of the UnlinkedData object generated by this line of the CSV.
        """
        ...

    def requires_mapping(self) -> bool:
        """
        Indicates whether the data source can undergo
        rename events/is tied to equities.
        
        :returns: true.
        """
        ...

    def supported_resolutions(self) -> typing.List[QuantConnect.Resolution]:
        """
        Gets a list of all the supported Resolutions
        
        :returns: All resolutions.
        """
        ...


class LinkedData(QuantConnect.Data.BaseData):
    """Data source that is linked (tickers that can have renames or be delisted)"""

    @property
    def count(self) -> int:
        """Example data"""
        ...

    @count.setter
    def count(self, value: int) -> None:
        ...

    def data_time_zone(self) -> typing.Any:
        """
        Set the data time zone to UTC
        
        :returns: Time zone as UTC.
        """
        ...

    def default_resolution(self) -> QuantConnect.Resolution:
        """
        Sets the default resolution to Second
        
        :returns: Resolution.Second.
        """
        ...

    def get_source(self, config: QuantConnect.Data.SubscriptionDataConfig, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.SubscriptionDataSource:
        """Return the URL string source of the file. This will be converted to a stream"""
        ...

    def is_sparse_data(self) -> bool:
        """
        Indicates whether the data source is sparse.
        If false, it will disable missing file logging.
        
        :returns: true.
        """
        ...

    def reader(self, config: QuantConnect.Data.SubscriptionDataConfig, line: str, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.BaseData:
        """
        Reader converts each line of the data source into BaseData objects. Each data type creates its own factory method, and returns a new instance of the object
        each time it is called. The returned object is assumed to be time stamped in the config.ExchangeTimeZone.
        """
        ...

    def requires_mapping(self) -> bool:
        """
        Indicates whether the data source can undergo
        rename events/is tied to equities.
        
        :returns: true.
        """
        ...

    def supported_resolutions(self) -> typing.List[QuantConnect.Resolution]:
        """
        Gets a list of all the supported Resolutions
        
        :returns: All resolutions.
        """
        ...


class UnlinkedDataTradeBar(QuantConnect.Data.Market.TradeBar):
    """Data source that is unlinked (no mapping) and takes any ticker when calling AddData"""

    any_ticker: bool
    """If true, we accept any ticker from the AddData call"""

    def __init__(self) -> None:
        """Creates a new instance of an UnlinkedTradeBar"""
        ...

    def data_time_zone(self) -> typing.Any:
        """
        Set the data time zone to UTC
        
        :returns: Time zone as UTC.
        """
        ...

    def default_resolution(self) -> QuantConnect.Resolution:
        """
        Sets the default resolution to Second
        
        :returns: Resolution.Second.
        """
        ...

    def get_source(self, config: QuantConnect.Data.SubscriptionDataConfig, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.SubscriptionDataSource:
        """
        Get Source for Custom Data File
        >> What source file location would you prefer for each type of usage:
        
        :param config: Configuration object
        :param date: Date of this source request if source spread across multiple files
        :param is_live_mode: true if we're in live mode, false for backtesting mode
        :returns: String source location of the file.
        """
        ...

    def is_sparse_data(self) -> bool:
        """
        Indicates whether the data source is sparse.
        If false, it will disable missing file logging.
        
        :returns: true.
        """
        ...

    def reader(self, config: QuantConnect.Data.SubscriptionDataConfig, line: str, date: datetime.datetime, is_live_mode: bool) -> QuantConnect.Data.BaseData:
        """
        Fetch the data from the storage and feed it line by line into the engine.
        
        :param config: Symbols, Resolution, DataType,
        :param line: Line from the data file requested
        :param date: Date of this reader request
        :param is_live_mode: true if we're in live mode, false for backtesting mode
        :returns: Enumerable iterator for returning each line of the required data.
        """
        ...

    def requires_mapping(self) -> bool:
        """
        Indicates whether the data source can undergo
        rename events/is tied to equities.
        
        :returns: true.
        """
        ...

    def supported_resolutions(self) -> typing.List[QuantConnect.Resolution]:
        """
        Gets a list of all the supported Resolutions
        
        :returns: All resolutions.
        """
        ...





from typing import overload
from enum import IntEnum
import abc
import typing

import System
import System.Collections
import System.Collections.Specialized

System_Collections_Specialized__EventContainer_Callable = typing.TypeVar("System_Collections_Specialized__EventContainer_Callable")
System_Collections_Specialized__EventContainer_ReturnType = typing.TypeVar("System_Collections_Specialized__EventContainer_ReturnType")


class NotifyCollectionChangedAction(IntEnum):
    """This class has no documentation."""

    ADD = 0

    REMOVE = 1

    REPLACE = 2

    MOVE = 3

    RESET = 4


class NotifyCollectionChangedEventArgs(System.EventArgs):
    """This class has no documentation."""

    @property
    def action(self) -> System.Collections.Specialized.NotifyCollectionChangedAction:
        ...

    @property
    def new_items(self) -> typing.List[typing.Any]:
        ...

    @property
    def old_items(self) -> typing.List[typing.Any]:
        ...

    @property
    def new_starting_index(self) -> int:
        ...

    @property
    def old_starting_index(self) -> int:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, changed_item: typing.Any) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, changed_item: typing.Any, index: int) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, new_item: typing.Any, old_item: typing.Any) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, new_item: typing.Any, old_item: typing.Any, index: int) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, changed_item: typing.Any, index: int, old_index: int) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, changed_items: System.Collections.IList) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, changed_items: System.Collections.IList, starting_index: int) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, new_items: System.Collections.IList, old_items: System.Collections.IList) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, new_items: System.Collections.IList, old_items: System.Collections.IList, starting_index: int) -> None:
        ...

    @overload
    def __init__(self, action: System.Collections.Specialized.NotifyCollectionChangedAction, changed_items: System.Collections.IList, index: int, old_index: int) -> None:
        ...


class INotifyCollectionChanged(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    @abc.abstractmethod
    def collection_changed(self) -> _EventContainer[typing.Callable[[System.Object, System.Collections.Specialized.NotifyCollectionChangedEventArgs], typing.Any], typing.Any]:
        ...

    @collection_changed.setter
    def collection_changed(self, value: _EventContainer[typing.Callable[[System.Object, System.Collections.Specialized.NotifyCollectionChangedEventArgs], typing.Any], typing.Any]) -> None:
        ...


class _EventContainer(typing.Generic[System_Collections_Specialized__EventContainer_Callable, System_Collections_Specialized__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> System_Collections_Specialized__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: System_Collections_Specialized__EventContainer_Callable) -> typing.Self:
        """Registers an event handler."""
        ...

    def __isub__(self, item: System_Collections_Specialized__EventContainer_Callable) -> typing.Self:
        """Unregisters an event handler."""
        ...





from typing import overload
from enum import IntEnum
import abc
import datetime
import typing

import QuantConnect.Api
import QuantConnect.Brokerages.Authentication
import System
import System.Threading
import System.Threading.Tasks

DelegatingHandler = typing.Any
AuthenticationHeaderValue = typing.Any
HttpResponseMessage = typing.Any

QuantConnect_Brokerages_Authentication_OAuthTokenHandler_TRequest = typing.TypeVar("QuantConnect_Brokerages_Authentication_OAuthTokenHandler_TRequest")
QuantConnect_Brokerages_Authentication_OAuthTokenHandler_TResponse = typing.TypeVar("QuantConnect_Brokerages_Authentication_OAuthTokenHandler_TResponse")


class AccessTokenMetaDataRequest(System.Object, metaclass=abc.ABCMeta):
    """Represents the base request for obtaining an access token, including brokerage and account information."""

    @property
    def brokerage(self) -> str:
        """
        Gets the name of the brokerage associated with the access token request.
        The value is normalized to lowercase.
        """
        ...

    @property
    def account_id(self) -> str:
        """Gets the account identifier (e.g., account number) associated with the brokerage."""
        ...

    def __init__(self, brokerage: str, account_id: str) -> None:
        """
        Initializes a new instance of the AccessTokenMetaDataRequest class.
        
        
        This codeEntityType is protected.
        
        :param brokerage: The name of the brokerage making the request. Will be normalized to lowercase.
        :param account_id: The account number or identifier associated with the brokerage.
        """
        ...

    def to_json(self) -> str:
        """
        Serializes the request into a compact JSON string with camelCase property naming.
        
        :returns: A JSON string representing the current request.
        """
        ...


class TokenType(IntEnum):
    """Defines the supported types of access tokens used for authentication."""

    BEARER = 0
    """A Bearer token, typically used for standard HTTP Authorization headers."""

    SESSION_TOKEN = 1
    """A Session token, typically used for username/password authorization headers."""


class TokenCredentials(System.Object):
    """
    Represents credentials required for token-based authentication,
    including the access token and its type (e.g., Bearer).
    """

    @property
    def token_type(self) -> QuantConnect.Brokerages.Authentication.TokenType:
        """Gets the type of the token (e.g., Bearer)."""
        ...

    @property
    def access_token(self) -> str:
        """Gets the token string used for authentication."""
        ...

    def __init__(self, token_type: QuantConnect.Brokerages.Authentication.TokenType, access_token: str) -> None:
        """
        Initializes a new instance of the TokenCredentials class.
        
        :param token_type: The type of the token.
        :param access_token: The token string.
        """
        ...


class TokenHandler(DelegatingHandler, metaclass=abc.ABCMeta):
    """
    Provides base functionality for token-based HTTP request handling,
    including automatic retries and token refresh on unauthorized responses.
    """

    def __init__(self, create_auth_header: typing.Callable[[QuantConnect.Brokerages.Authentication.TokenType, str], AuthenticationHeaderValue] = None, retry_interval: typing.Optional[datetime.timedelta] = None) -> None:
        """
        Initializes a new instance of the TokenHandler class.
        
        
        This codeEntityType is protected.
        
        :param create_auth_header: An optional delegate for creating an AuthenticationHeaderValue
        from the token type and access token. If not provided, a default implementation is used.
        :param retry_interval: An optional time interval to wait between retry attempts when fetching the token or retrying a failed request.
        If null, the default interval of 5 seconds is used.
        """
        ...

    def get_access_token(self, cancellation_token: System.Threading.CancellationToken) -> QuantConnect.Brokerages.Authentication.TokenCredentials:
        """
        Retrieves a valid access token for authenticating HTTP requests.
        Must be implemented by derived classes to provide token type and value,
        with optional support for caching and refresh logic.
        
        :param cancellation_token: A cancellation token that can be used to cancel the token retrieval operation.
        :returns: A TokenCredentials instance containing the token type and access token string.
        """
        ...

    def send(self, request: typing.Any, cancellation_token: System.Threading.CancellationToken) -> typing.Any:
        """
        Sends an HTTP request synchronously with retry support.
        This override includes token-based authentication and refresh logic on 401 Unauthorized responses.
        
        
        This codeEntityType is protected.
        
        :param request: The HTTP request message to send.
        :param cancellation_token: A cancellation token to cancel operation.
        :returns: The HTTP response message.
        """
        ...

    def send_async(self, request: typing.Any, cancellation_token: System.Threading.CancellationToken) -> System.Threading.Tasks.Task[HttpResponseMessage]:
        """
        Sends an HTTP request asynchronously by internally invoking the synchronous send(HttpRequestMessage, CancellationToken) method.
        This is useful for compatibility with components that require an asynchronous pipeline, even though the core logic is synchronous.
        
        
        This codeEntityType is protected.
        
        :param request: The HTTP request message to send.
        :param cancellation_token: A cancellation token to cancel the operation.
        :returns: A task representing the asynchronous operation, containing the HTTP response message.
        """
        ...


class OAuthTokenHandler(typing.Generic[QuantConnect_Brokerages_Authentication_OAuthTokenHandler_TRequest, QuantConnect_Brokerages_Authentication_OAuthTokenHandler_TResponse], QuantConnect.Brokerages.Authentication.TokenHandler):
    """
    Handles OAuth token retrieval and caching by interacting with the Lean platform.
    Implements retry and expiration logic for secure HTTP communication.
    """

    def __init__(self, api_client: QuantConnect.Api.ApiConnection, model_request: QuantConnect_Brokerages_Authentication_OAuthTokenHandler_TRequest) -> None:
        """
        Initializes a new instance of the OAuthTokenHandler{TRequest, TResponse} class.
        
        :param api_client: The API client used to communicate with the Lean platform.
        :param model_request: The request model used to generate the access token.
        """
        ...

    def get_access_token(self, cancellation_token: System.Threading.CancellationToken) -> QuantConnect.Brokerages.Authentication.TokenCredentials:
        """
        Retrieves a valid access token from the Lean platform.
        Caches and reuses tokens until expiration to minimize unnecessary requests.
        
        :param cancellation_token: A token used to observe cancellation requests.
        :returns: A tuple containing the token type and access token string.
        """
        ...


class AccessTokenMetaDataResponse(QuantConnect.Api.RestResponse, metaclass=abc.ABCMeta):
    """Represents a response containing metadata about an access token issued by Lean."""

    @property
    def access_token(self) -> str:
        """Gets the access token provided by Lean."""
        ...

    @property
    def token_type(self) -> QuantConnect.Brokerages.Authentication.TokenType:
        """Gets the type of the token (e.g., "Bearer")."""
        ...

    @property
    def expiration(self) -> datetime.datetime:
        """Gets the UTC expiration timestamp of the access token, with a 1-minute safety buffer applied."""
        ...

    @expiration.setter
    def expiration(self, value: datetime.datetime) -> None:
        ...

    def __init__(self, access_token: str, token_type: QuantConnect.Brokerages.Authentication.TokenType, expires: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Initializes a new instance of the AccessTokenMetaDataResponse class.
        
        
        This codeEntityType is protected.
        
        :param access_token: The access token string provided by the authentication service.
        :param token_type: The type of the token (e.g., Bearer).
        :param expires: The expiration time of the access token (in UTC), with safety buffer applied.
        """
        ...





from typing import overload
from enum import IntEnum
import System
import System.Net.Security


class SslPolicyErrors(IntEnum):
    """This class has no documentation."""

    NONE = ...

    REMOTE_CERTIFICATE_NOT_AVAILABLE = ...

    REMOTE_CERTIFICATE_NAME_MISMATCH = ...

    REMOTE_CERTIFICATE_CHAIN_ERRORS = ...


class AuthenticationLevel(IntEnum):
    """This class has no documentation."""

    NONE = 0

    MUTUAL_AUTH_REQUESTED = 1

    MUTUAL_AUTH_REQUIRED = 2





from typing import overload
from enum import IntEnum
import typing

import System
import System.Net.Cache


class RequestCacheLevel(IntEnum):
    """This class has no documentation."""

    DEFAULT = 0

    BYPASS_CACHE = 1

    CACHE_ONLY = 2

    CACHE_IF_AVAILABLE = 3

    REVALIDATE = 4

    RELOAD = 5

    NO_CACHE_NO_STORE = 6


class RequestCachePolicy(System.Object):
    """This class has no documentation."""

    @property
    def level(self) -> System.Net.Cache.RequestCacheLevel:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, level: System.Net.Cache.RequestCacheLevel) -> None:
        ...

    def to_string(self) -> str:
        ...





from typing import overload
from enum import IntEnum
import typing

import System
import System.Runtime.InteropServices
import System.Runtime.InteropServices.Marshalling

System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T = typing.TypeVar("System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T")
System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_TUnmanagedElement = typing.TypeVar("System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_TUnmanagedElement")
System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T = typing.TypeVar("System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T")
System_Runtime_InteropServices_Marshalling_ArrayMarshaller_TUnmanagedElement = typing.TypeVar("System_Runtime_InteropServices_Marshalling_ArrayMarshaller_TUnmanagedElement")
System_Runtime_InteropServices_Marshalling_SpanMarshaller_T = typing.TypeVar("System_Runtime_InteropServices_Marshalling_SpanMarshaller_T")
System_Runtime_InteropServices_Marshalling_SpanMarshaller_TUnmanagedElement = typing.TypeVar("System_Runtime_InteropServices_Marshalling_SpanMarshaller_TUnmanagedElement")
System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_T = typing.TypeVar("System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_T")
System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_TUnmanagedElement = typing.TypeVar("System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_TUnmanagedElement")
System_Runtime_InteropServices_Marshalling_SafeHandleMarshaller_T = typing.TypeVar("System_Runtime_InteropServices_Marshalling_SafeHandleMarshaller_T")


class AnsiStringMarshaller(System.Object):
    """This class has no documentation."""

    class ManagedToUnmanagedIn:
        """This class has no documentation."""

        BUFFER_SIZE: int

        def free(self) -> None:
            ...

        def from_managed(self, managed: str, buffer: System.Span[int]) -> None:
            ...

        def to_unmanaged(self) -> typing.Any:
            ...

    @staticmethod
    def convert_to_managed(unmanaged: typing.Any) -> str:
        ...

    @staticmethod
    def convert_to_unmanaged(managed: str) -> typing.Any:
        ...

    @staticmethod
    def free(unmanaged: typing.Any) -> None:
        ...


class MarshalMode(IntEnum):
    """This class has no documentation."""

    DEFAULT = 0

    MANAGED_TO_UNMANAGED_IN = 1

    MANAGED_TO_UNMANAGED_REF = 2

    MANAGED_TO_UNMANAGED_OUT = 3

    UNMANAGED_TO_MANAGED_IN = 4

    UNMANAGED_TO_MANAGED_REF = 5

    UNMANAGED_TO_MANAGED_OUT = 6

    ELEMENT_IN = 7

    ELEMENT_REF = 8

    ELEMENT_OUT = 9


class ReadOnlySpanMarshaller(typing.Generic[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T, System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_TUnmanagedElement], System.Object):
    """This class has no documentation."""

    class UnmanagedToManagedOut(System.Object):
        """This class has no documentation."""

        @staticmethod
        def allocate_container_for_unmanaged_elements(managed: System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T], num_elements: typing.Optional[int]) -> typing.Tuple[typing.Any, int]:
            ...

        @staticmethod
        def get_managed_values_source(managed: System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T]) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T]:
            ...

        @staticmethod
        def get_unmanaged_values_destination(unmanaged: typing.Any, num_elements: int) -> System.Span[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_TUnmanagedElement]:
            ...

    class ManagedToUnmanagedIn:
        """This class has no documentation."""

        BUFFER_SIZE: int

        def free(self) -> None:
            ...

        def from_managed(self, managed: System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T], buffer: System.Span[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_TUnmanagedElement]) -> None:
            ...

        def get_managed_values_source(self) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T]:
            ...

        @overload
        def get_pinnable_reference(self) -> typing.Any:
            ...

        @staticmethod
        @overload
        def get_pinnable_reference(managed: System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T]) -> typing.Any:
            ...

        def get_unmanaged_values_destination(self) -> System.Span[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_TUnmanagedElement]:
            ...

        def to_unmanaged(self) -> typing.Any:
            ...

    class ManagedToUnmanagedOut:
        """This class has no documentation."""

        def free(self) -> None:
            ...

        def from_unmanaged(self, unmanaged: typing.Any) -> None:
            ...

        def get_managed_values_destination(self, num_elements: int) -> System.Span[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T]:
            ...

        def get_unmanaged_values_source(self, num_elements: int) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_TUnmanagedElement]:
            ...

        def to_managed(self) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ReadOnlySpanMarshaller_T]:
            ...


class ComVariant(System.IDisposable):
    """This class has no documentation."""

    NULL: System.Runtime.InteropServices.Marshalling.ComVariant

    @property
    def var_type(self) -> System.Runtime.InteropServices.VarEnum:
        ...

    def dispose(self) -> None:
        ...


class ArrayMarshaller(typing.Generic[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T, System_Runtime_InteropServices_Marshalling_ArrayMarshaller_TUnmanagedElement], System.Object):
    """This class has no documentation."""

    class ManagedToUnmanagedIn:
        """This class has no documentation."""

        BUFFER_SIZE: int

        def free(self) -> None:
            ...

        def from_managed(self, array: typing.List[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T], buffer: System.Span[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_TUnmanagedElement]) -> None:
            ...

        def get_managed_values_source(self) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T]:
            ...

        @overload
        def get_pinnable_reference(self) -> typing.Any:
            ...

        @staticmethod
        @overload
        def get_pinnable_reference(array: typing.List[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T]) -> typing.Any:
            ...

        def get_unmanaged_values_destination(self) -> System.Span[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_TUnmanagedElement]:
            ...

        def to_unmanaged(self) -> typing.Any:
            ...

    @staticmethod
    def allocate_container_for_managed_elements(unmanaged: typing.Any, num_elements: int) -> typing.List[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T]:
        ...

    @staticmethod
    def allocate_container_for_unmanaged_elements(managed: typing.List[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T], num_elements: typing.Optional[int]) -> typing.Tuple[typing.Any, int]:
        ...

    @staticmethod
    def free(unmanaged: typing.Any) -> None:
        ...

    @staticmethod
    def get_managed_values_destination(managed: typing.List[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T]) -> System.Span[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T]:
        ...

    @staticmethod
    def get_managed_values_source(managed: typing.List[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T]) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_T]:
        ...

    @staticmethod
    def get_unmanaged_values_destination(unmanaged: typing.Any, num_elements: int) -> System.Span[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_TUnmanagedElement]:
        ...

    @staticmethod
    def get_unmanaged_values_source(unmanaged_value: typing.Any, num_elements: int) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_ArrayMarshaller_TUnmanagedElement]:
        ...


class CustomMarshallerAttribute(System.Attribute):
    """This class has no documentation."""

    class GenericPlaceholder:
        """This class has no documentation."""

    @property
    def managed_type(self) -> typing.Type:
        ...

    @property
    def marshal_mode(self) -> System.Runtime.InteropServices.Marshalling.MarshalMode:
        ...

    @property
    def marshaller_type(self) -> typing.Type:
        ...

    def __init__(self, managed_type: typing.Type, marshal_mode: System.Runtime.InteropServices.Marshalling.MarshalMode, marshaller_type: typing.Type) -> None:
        ...


class SpanMarshaller(typing.Generic[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T, System_Runtime_InteropServices_Marshalling_SpanMarshaller_TUnmanagedElement], System.Object):
    """This class has no documentation."""

    class ManagedToUnmanagedIn:
        """This class has no documentation."""

        BUFFER_SIZE: int

        def free(self) -> None:
            ...

        def from_managed(self, managed: System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T], buffer: System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_TUnmanagedElement]) -> None:
            ...

        def get_managed_values_source(self) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T]:
            ...

        @overload
        def get_pinnable_reference(self) -> typing.Any:
            ...

        @staticmethod
        @overload
        def get_pinnable_reference(managed: System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T]) -> typing.Any:
            ...

        def get_unmanaged_values_destination(self) -> System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_TUnmanagedElement]:
            ...

        def to_unmanaged(self) -> typing.Any:
            ...

    @staticmethod
    def allocate_container_for_managed_elements(unmanaged: typing.Any, num_elements: int) -> System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T]:
        ...

    @staticmethod
    def allocate_container_for_unmanaged_elements(managed: System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T], num_elements: typing.Optional[int]) -> typing.Tuple[typing.Any, int]:
        ...

    @staticmethod
    def free(unmanaged: typing.Any) -> None:
        ...

    @staticmethod
    def get_managed_values_destination(managed: System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T]) -> System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T]:
        ...

    @staticmethod
    def get_managed_values_source(managed: System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T]) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_SpanMarshaller_T]:
        ...

    @staticmethod
    def get_unmanaged_values_destination(unmanaged: typing.Any, num_elements: int) -> System.Span[System_Runtime_InteropServices_Marshalling_SpanMarshaller_TUnmanagedElement]:
        ...

    @staticmethod
    def get_unmanaged_values_source(unmanaged: typing.Any, num_elements: int) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_SpanMarshaller_TUnmanagedElement]:
        ...


class NativeMarshallingAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def native_type(self) -> typing.Type:
        ...

    def __init__(self, native_type: typing.Type) -> None:
        ...


class BStrStringMarshaller(System.Object):
    """This class has no documentation."""

    class ManagedToUnmanagedIn:
        """This class has no documentation."""

        BUFFER_SIZE: int

        def free(self) -> None:
            ...

        def from_managed(self, managed: str, buffer: System.Span[int]) -> None:
            ...

        def to_unmanaged(self) -> typing.Any:
            ...

    @staticmethod
    def convert_to_managed(unmanaged: typing.Any) -> str:
        ...

    @staticmethod
    def convert_to_unmanaged(managed: str) -> typing.Any:
        ...

    @staticmethod
    def free(unmanaged: typing.Any) -> None:
        ...


class PointerArrayMarshaller(typing.Generic[System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_T, System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_TUnmanagedElement], System.Object):
    """This class has no documentation."""

    class ManagedToUnmanagedIn:
        """This class has no documentation."""

        BUFFER_SIZE: int

        def free(self) -> None:
            ...

        def from_managed(self, array: typing.List[typing.Any], buffer: System.Span[System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_TUnmanagedElement]) -> None:
            ...

        def get_managed_values_source(self) -> System.ReadOnlySpan[System.IntPtr]:
            ...

        @overload
        def get_pinnable_reference(self) -> typing.Any:
            ...

        @staticmethod
        @overload
        def get_pinnable_reference(array: typing.List[typing.Any]) -> typing.Any:
            ...

        def get_unmanaged_values_destination(self) -> System.Span[System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_TUnmanagedElement]:
            ...

        def to_unmanaged(self) -> typing.Any:
            ...

    @staticmethod
    def allocate_container_for_managed_elements(unmanaged: typing.Any, num_elements: int) -> typing.List[typing.Any]:
        ...

    @staticmethod
    def allocate_container_for_unmanaged_elements(managed: typing.List[typing.Any], num_elements: typing.Optional[int]) -> typing.Tuple[typing.Any, int]:
        ...

    @staticmethod
    def free(unmanaged: typing.Any) -> None:
        ...

    @staticmethod
    def get_managed_values_destination(managed: typing.List[typing.Any]) -> System.Span[System.IntPtr]:
        ...

    @staticmethod
    def get_managed_values_source(managed: typing.List[typing.Any]) -> System.ReadOnlySpan[System.IntPtr]:
        ...

    @staticmethod
    def get_unmanaged_values_destination(unmanaged: typing.Any, num_elements: int) -> System.Span[System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_TUnmanagedElement]:
        ...

    @staticmethod
    def get_unmanaged_values_source(unmanaged_value: typing.Any, num_elements: int) -> System.ReadOnlySpan[System_Runtime_InteropServices_Marshalling_PointerArrayMarshaller_TUnmanagedElement]:
        ...


class SafeHandleMarshaller(typing.Generic[System_Runtime_InteropServices_Marshalling_SafeHandleMarshaller_T], System.Object):
    """This class has no documentation."""

    class ManagedToUnmanagedIn:
        """This class has no documentation."""

        def free(self) -> None:
            ...

        def from_managed(self, handle: System_Runtime_InteropServices_Marshalling_SafeHandleMarshaller_T) -> None:
            ...

        def to_unmanaged(self) -> System.IntPtr:
            ...

    class ManagedToUnmanagedRef:
        """This class has no documentation."""

        def __init__(self) -> None:
            ...

        def free(self) -> None:
            ...

        def from_managed(self, handle: System_Runtime_InteropServices_Marshalling_SafeHandleMarshaller_T) -> None:
            ...

        def from_unmanaged(self, value: System.IntPtr) -> None:
            ...

        def on_invoked(self) -> None:
            ...

        def to_managed_finally(self) -> System_Runtime_InteropServices_Marshalling_SafeHandleMarshaller_T:
            ...

        def to_unmanaged(self) -> System.IntPtr:
            ...

    class ManagedToUnmanagedOut:
        """This class has no documentation."""

        def __init__(self) -> None:
            ...

        def free(self) -> None:
            ...

        def from_unmanaged(self, value: System.IntPtr) -> None:
            ...

        def to_managed(self) -> System_Runtime_InteropServices_Marshalling_SafeHandleMarshaller_T:
            ...


class Utf8StringMarshaller(System.Object):
    """This class has no documentation."""

    class ManagedToUnmanagedIn:
        """This class has no documentation."""

        BUFFER_SIZE: int

        def free(self) -> None:
            ...

        def from_managed(self, managed: str, buffer: System.Span[int]) -> None:
            ...

        def to_unmanaged(self) -> typing.Any:
            ...

    @staticmethod
    def convert_to_managed(unmanaged: typing.Any) -> str:
        ...

    @staticmethod
    def convert_to_unmanaged(managed: str) -> typing.Any:
        ...

    @staticmethod
    def free(unmanaged: typing.Any) -> None:
        ...


class MarshalUsingAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def native_type(self) -> typing.Type:
        ...

    @property
    def count_element_name(self) -> str:
        ...

    @count_element_name.setter
    def count_element_name(self, value: str) -> None:
        ...

    @property
    def constant_element_count(self) -> int:
        ...

    @constant_element_count.setter
    def constant_element_count(self, value: int) -> None:
        ...

    @property
    def element_indirection_depth(self) -> int:
        ...

    @element_indirection_depth.setter
    def element_indirection_depth(self, value: int) -> None:
        ...

    RETURNS_COUNT_VALUE: str = "return-value"

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, native_type: typing.Type) -> None:
        ...


class ContiguousCollectionMarshallerAttribute(System.Attribute):
    """This class has no documentation."""


class Utf16StringMarshaller(System.Object):
    """This class has no documentation."""

    @staticmethod
    def convert_to_managed(unmanaged: typing.Any) -> str:
        ...

    @staticmethod
    def convert_to_unmanaged(managed: str) -> typing.Any:
        ...

    @staticmethod
    def free(unmanaged: typing.Any) -> None:
        ...

    @staticmethod
    def get_pinnable_reference(str: str) -> typing.Any:
        ...





from typing import overload
from enum import IntEnum
import typing

import QuantConnect.Lean.Launcher
import System


class Program(System.Object):
    """This class has no documentation."""

    @staticmethod
    def exit(exit_code: int) -> None:
        ...

    @staticmethod
    def exit_key_press(sender: typing.Any, args: System.ConsoleCancelEventArgs) -> None:
        ...


